--
-- Copyright(C) 2006-2010 Masaharu Nomachi (Osaka University) All Rights reserved.
--
-- Copyright(C) 2007-2010 Shimafuji Electric Inc. All Rights reserved.
--
-- Shimafuji Electric Inc. grants permission to use, copy, redistribute and/or modify
-- this souce file or products generated by this source file without restriction
-- provided that above copyright statement, this list of conditions and the following
-- disclaimer is preserved and that any derivative work contains the original
-- copyright notice.
-- This source file is distributed WITHOUT ANY WARRANTY, without even the implied
-- warranty of merchantability or fitness for a particular purpose.
-- By providing this source file Shimafuji Electric Inc. making no representation
-- that this source file or products generated by this source file is free from
-- any claims of infringement.
--
---- Update history ----
--
-- IP-Rev1.04 2011/04/06 Changes to the "spw_if.vhd".
--

library IEEE;
use IEEE.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity spw_receiver_sync is
--entity spw_rx is
	generic (
		DISCON_COUNT : integer := 141		-- 141@166MHz=850ns
	);
	port (
		SpW_S : in  std_logic;
		SpW_D : in  std_logic;
--
		Rx_data : out std_logic_vector (8 downto 0);
		Rx_ds : out std_logic;
--
		Rx_time : out std_logic_vector (7 downto 0);
		Rx_ts : out std_logic;
--
		Rx_Nchar : out std_logic;
		Rx_fct : out std_logic;
		Rx_null	: out std_logic;
		Rx_eep : out std_logic;
		Rx_eop : out std_logic;
--
		Rx_off : out std_logic;
		Credit_error : out std_logic;
		Rx_error : out std_logic;
		parity_error : out std_logic;		-- debug
		escape_error : out std_logic;		-- debug
		disconnect_error : out std_logic;	-- debug
		Rx_reset : in  std_logic;
		
		buf_write : out std_logic;
		buf_rdy : in std_logic;
--
		enable_rx : in std_logic;
		clock : in std_logic
	);
end;

architecture RTL of spw_receiver_sync is

	signal data_reg  	: std_logic_vector(11 downto 0);
	signal parity		: std_logic;
	signal ESC_flag 	: std_logic;
	signal SpW_sync		: std_logic_vector(1 downto 0);
	signal bit_count	: std_logic_vector(3 downto 0);
	signal t : std_logic_vector (7 downto 0);
	signal time_out_reg, parity_error_reg : std_logic;
	signal escape_error_reg : std_logic;
	signal C_flag, D_flag 	: std_logic;
	
	TYPE ss_type IS (
			idle, spw_off, 
			even_0, even_1, wait_even, odd_0, odd_1, wait_odd
		);
	SIGNAL ss : ss_type;
	
	signal Rx_eop_reg : std_logic;
	signal Rx_eep_reg : std_logic;
	signal Rx_ds_reg : std_logic;

begin
	process (clock) begin
	if (clock'event and clock = '1') then
	SpW_sync <= SpW_S & SpW_D;
	end if;
	end process;
	
	PROCESS (clock)
	BEGIN
		if Rx_reset = '1' then
			ss <= idle;
		elsif clock'EVENT and clock = '1' then
	--		SpW_sync <= SpW_S & SpW_D;
			if ss = idle then
				if SpW_sync = "00" then
					ss <= spw_off; 
				end if;
			elsif ss = spw_off then
				if SpW_sync = "10" then
					ss <= odd_0;
				end if;
		-----
			elsif ss = even_1 or ss = even_0 or ss = wait_odd then
				if SpW_sync = "10" then
					ss <= odd_0;
				elsif SpW_sync = "01" then
					ss <= odd_1;
				else
					ss <= wait_odd;
				end if;
			elsif ss = odd_1 or ss = odd_0 or ss = wait_even then
				if SpW_sync = "00" then
					ss <= even_0;
				elsif SpW_sync = "11" then
					ss <= even_1;
				else
					ss <= wait_even;
				end if;
			end if;
		end if;
	end process;
	
	PROCESS (clock)
	BEGIN
		if clock'EVENT and clock = '1' then
		-- data register --
			if ss=spw_off then
				data_reg <= X"000";
			elsif ss=odd_1 or ss=even_1 then
				data_reg <= '1' & data_reg(11 downto 1);
			elsif ss=odd_0 or ss=even_0 then
				data_reg <= '0' & data_reg(11 downto 1);
			end if;
		-- parity --
			--if ss=spw_off then
			--	parity <= '0';
			--elsif ss=odd_1 or ss=even_1 then
			--	parity <= not parity;
			--end if;
			--if ss=idle then
			--	parity_error <= '0';
			
			if ss=idle then
				parity_error_reg <= '0';
			elsif ss=spw_off then
				parity <= '0';
			elsif bit_count=0 and ss=even_1 then
				if parity = '1' then
					parity_error_reg <= '1';
					parity <= '0';
				end if;
			elsif bit_count=0 and ss=even_0 then
				if parity='0' then
					parity_error_reg <= '1';
				else
					parity <= '0';
				end if;
			elsif ss=odd_1 or ss=even_1 then
				parity <= not parity;
			end if;
		-- off timer --
			if ss=idle then
				t <= X"00";
				time_out_reg <= '0';
			elsif ss=odd_1 or ss=even_1 or ss=odd_0 or ss=even_0 then
				t <= X"00";
			elsif	ss=wait_odd or ss=wait_even then
				if (t < DISCON_COUNT) then
					t <= t + 1;
				else
					time_out_reg <= '1';
				end if;
			end if;
			--time_out <= t(7);
		-- data/control flag --
			if ss=idle then 
				C_flag <= '0'; D_flag <= '0';
			elsif bit_count=0 and ss=even_0 then
				C_flag <= '0'; D_flag <= '1';
			elsif bit_count=0 and ss=even_1 then
				C_flag <= '1'; D_flag <= '0';
			end if;
		-- bit count --
			if ss=idle or ss=spw_off then
				bit_count <= X"0";
			elsif ss=even_1 or ss=even_0 then
				if bit_count=1 and C_flag='1' then
					bit_count <= X"0";
				elsif bit_count=4 and C_flag='0' then
					bit_count <= X"0";
				else
					bit_count <= bit_count+1;
				end if;
			end if;
		-- data / time code --
			if bit_count=0 and (ss=odd_0 or ss=odd_1) then
				if D_flag='1' then
					if ESC_flag='1' then
						Rx_time <= data_reg(11 downto 4);
					else
						Rx_data <= '0' & data_reg(11 downto 4);
						buf_write <= '1';
					end if;
				elsif C_flag='1' then
					Rx_data <= '1' & "000000" & data_reg(11 downto 10);
					if (data_reg (11 downto 10) = "10" or data_reg (11 downto 10) = "01") then
						buf_write <= '1';
					end if;
				end if;
			else
				buf_write <= '0';
			end if;
		-- strobe --
			if bit_count=0 and (ss=odd_0 or ss=odd_1) then
				if C_flag='1' then
					if data_reg(11 downto 10)="00" then -- FCT
						if ESC_flag='1' then
							Rx_null <= '1';
							ESC_flag <= '0';
						else
							Rx_fct <= '1';
						end if;
					elsif data_reg(11 downto 10)="11" then -- ESC
						if ESC_flag = '1' then
							escape_error_reg <= '1';
						else
							ESC_flag <= '1';
						end if;
					elsif data_reg(11 downto 10)="10" then -- EOP
						if ESC_flag = '1' then
							escape_error_reg <= '1';
						else
							Rx_eop_reg <='1';--	ESC_flag <= '0';
						end if;
					elsif data_reg(11 downto 10)="01" then -- EEP
						if ESC_flag = '1' then
							escape_error_reg <= '1';
						else
							Rx_eep_reg <='1';--  ESC_flag <= '0';
						end if;
					end if;
				elsif D_flag='1' then
					if ESC_flag='1' then
						Rx_ts <= '1'; ESC_flag <= '0';
					else
						Rx_ds_reg <= '1';
					--	if (buf_rdy = '0') then
					--		Credit_error <= '1';
					--	end if;
					end if;
				end if;
			elsif (bit_count=1 and (ss=odd_0 or ss=odd_1)) then
				Rx_ds_reg <='0';
				Rx_ts <='0';
				Rx_null <='0';
				Rx_fct <='0';
				Rx_eop_reg <='0';
				Rx_eep_reg <='0';
			elsif ss=idle then
				Rx_ds_reg <='0';
				Rx_ts <='0';
				Rx_null <='0';
				Rx_fct <='0';
				Rx_eop_reg <='0';
				Rx_eep_reg <='0';
				Credit_error <= '0';
				escape_error_reg <= '0';
				ESC_flag <= '0';
			end if;
		end if;
	end process;

	Rx_off <= '1' when SS=spw_off else '0';
	Rx_error <= '1' when time_out_reg='1' or parity_error_reg='1' or escape_error_reg = '1' else '0';
	Rx_Nchar <= '1' when Rx_eop_reg = '1' or Rx_eep_reg = '1' or Rx_ds_reg = '1' else '0';
	Rx_ds <= Rx_ds_reg;
	Rx_eop <= Rx_eop_reg;
	Rx_eep <= Rx_eep_reg;
	
	parity_error <= parity_error_reg;
	escape_error <= escape_error_reg;
	disconnect_error <= time_out_reg;

end RTL;