--
-- Copyright(C) 2007-2010 Shimafuji Electric Inc. All Rights reserved.
--
-- Shimafuji Electric Inc. grants permission to use, copy, redistribute and/or modify
-- this souce file or products generated by this source file without restriction
-- provided that above copyright statement, this list of conditions and the following
-- disclaimer is preserved and that any derivative work contains the original
-- copyright notice.
-- This source file is distributed WITHOUT ANY WARRANTY, without even the implied
-- warranty of merchantability or fitness for a particular purpose.
-- By providing this source file Shimafuji Electric Inc. making no representation
-- that this source file or products generated by this source file is free from
-- any claims of infringement.
--

----------------------------------------------------------------------------------
-- chapter 8 state machine
----------------------------------------------------------------------------------
--
-- Update history --

-- Ver1.00 --
-- 2007/12/12 disabled́ustatedȍ~ɑJڂȢAstartedȍ~resetɑJځAƉ߁v
-- 2007/12/12 resetԂ6.4usJEgȂs̏C
-- 2008/03/21 ErrorWaitԂNullMŃZbgԈႢ̏C
-- 2008/03/21 Sequence Error TimeCodeMǉ
-- 2008/05/28 Error recovery schemeǉ (fifo_preparedMǉ)
--

-- IP-Rev1.03 --
-- 2010/12/21 (bug fix.) correct state transition error of (ready,autostart) condition.
--
-- IP-Rev1.04 --
-- 2011/04/06 Changes to the "spw_if.vhd".
--

library IEEE; 
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity spw_state_machine is
	port (
		clk : in std_logic;
		reset : in std_logic;
		after128 : in std_logic;
		after64 : in std_logic;
		link_start : in std_logic;
		link_disable : in std_logic;
		auto_start : in std_logic;
		enable_tx : out std_logic;
		send_nulls : out std_logic;
		send_fcts : out std_logic;
		send_n_chars : out std_logic;
		send_time_codes : out std_logic;
		got_fct : in std_logic;
		got_time_code : in std_logic;
		got_n_char : in std_logic;
		got_null : in std_logic;
		got_bit : in std_logic;
		credit_err : in std_logic;
		rx_err : in std_logic;
		enable_rx : out std_logic;
		sequence_error : out std_logic;
		reset_out : out std_logic;
		fifo_prepared : in std_logic;
		
		reset64 : out std_logic;
		count128 : out std_logic
	);
end spw_state_machine;

architecture Behavioral of spw_state_machine is

	component sync_one_pulse is
		port (
			clk : in std_logic;
			reset : in std_logic;
			async_in : in std_logic;
			sync_out : out std_logic
		);
	end component;
	
	type lstate is (
		LS_ERRRESET,
		LS_ERRWAIT,
		LS_READY,
		LS_STARTED,
		LS_CONNECTING,
		LS_RUN
	);
	signal link_state : lstate;
	
	signal got_null_sync : std_logic;
	signal got_fct_sync : std_logic;
	signal got_time_code_sync : std_logic;
	signal got_n_char_sync : std_logic;
	
	signal async_error : std_logic;
	signal rx_errors : std_logic;
	
	signal seq_error_reg : std_logic;
	
begin

	
	got_null_pulse : sync_one_pulse
	port map (
		clk => clk,
		reset => reset,
		async_in => got_null,
		sync_out => got_null_sync
	);
	
	got_fct_pulse : sync_one_pulse
	port map (
		clk => clk,
		reset => reset,
		async_in => got_fct,
		sync_out => got_fct_sync
	);
	
	got_time_code_pulse : sync_one_pulse
	port map (
		clk => clk,
		reset => reset,
		async_in => got_time_code,
		sync_out => got_time_code_sync
	);
	
	got_n_char_pulse : sync_one_pulse
	port map (
		clk => clk,
		reset => reset,
		async_in => got_n_char,
		sync_out => got_n_char_sync
	);
	
	async_error <= rx_err or credit_err;
	error_pulse : sync_one_pulse
	port map (
		clk => clk,
		reset => reset,
		async_in => async_error,
		sync_out => rx_errors
	);
	
	sequence_error <= seq_error_reg;
	
	process (clk, reset)
	begin
		if (reset = '1') then
			link_state <= LS_ERRRESET;
			reset_out <= '1';
			enable_rx <= '0';
			enable_tx <= '0';
			send_nulls <= '0';
			send_fcts <= '0';
			send_n_chars <= '0';
			send_time_codes <= '0';
			
			seq_error_reg <= '0';
			
			reset64 <= '1';
			count128 <= '0';
			
		elsif (clk'event and clk = '1') then
		
			if (link_state = LS_ERRRESET) then
				if (fifo_prepared = '1') then
					reset64 <= '0';
				end if;
				reset_out <= '1';
				enable_rx <= '0';
				enable_tx <= '0';
				send_nulls <= '0';
				send_fcts <= '0';
				send_n_chars <= '0';
				send_time_codes <= '0';
				seq_error_reg <= '0';
				
				if (rx_errors = '1') then
					--reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (after64 = '1') then
					count128 <= '1';
					link_state <= LS_ERRWAIT;
				
				end if;
				
			elsif (link_state = LS_ERRWAIT) then
				reset_out <= '0';
				count128 <= '0';
				enable_rx <= '1';
				
				if (rx_errors = '1') then
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (got_time_code_sync = '1' or got_fct_sync = '1' or got_n_char_sync = '1') then
					-- character seqeunce error
					seq_error_reg <= '1';
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (after128 = '1') then
					link_state <= LS_READY;
				end if;
			
			elsif (link_state = LS_READY) then
				enable_rx <= '1';
				if (rx_errors = '1') then
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (got_fct_sync = '1' or got_n_char_sync = '1' or got_time_code_sync = '1') then
					-- character seqeunce error
					seq_error_reg <= '1';
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (auto_start = '1' and got_null_sync = '1') then
					count128 <= '1';
					link_state <= LS_STARTED;	-- Link Enabled
				elsif (link_start = '1') then
					count128 <= '1';
					link_state <= LS_STARTED;	-- Link Enabled
				end if;
			
			elsif (link_state = LS_STARTED) then
				enable_tx <= '1';
				enable_rx <= '1';
				send_nulls <= '1';
				count128 <= '0';
				
				if (rx_errors = '1') then
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (link_disable = '1') then
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (got_fct_sync = '1' or got_n_char_sync = '1' or got_time_code_sync = '1') then
					-- character seqeunce error
					seq_error_reg <= '1';
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (after128 = '1') then
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (got_null_sync = '1') then
					count128 <= '1';
					link_state <= LS_CONNECTING;
				end if;
			
			elsif (link_state = LS_CONNECTING) then
				count128 <= '0';
				enable_tx <= '1';
				enable_rx <= '1';
				send_fcts <= '1';
				
				if (rx_errors = '1') then
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (link_disable = '1') then
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (after128 = '1') then
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (got_n_char_sync = '1') then
					-- character seqeunce error
					seq_error_reg <= '1';
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (got_fct_sync = '1') then
					link_state <= LS_RUN;
				end if;
			
			elsif (link_state = LS_RUN) then
				enable_tx <= '1';
				enable_rx <= '1';
				send_n_chars <= '1';
				send_time_codes <= '1';
			
				if (rx_errors = '1') then
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				elsif (link_disable = '1') then
					reset64 <= '1';
					link_state <= LS_ERRRESET;
				end if;
				
			end if;
			
		end if;
	end process;
	
end Behavioral;
