--
-- Copyright(C) 2007-2010 Shimafuji Electric Inc. All Rights reserved.
--
-- Shimafuji Electric Inc. grants permission to use, copy, redistribute and/or modify
-- this souce file or products generated by this source file without restriction
-- provided that above copyright statement, this list of conditions and the following
-- disclaimer is preserved and that any derivative work contains the original
-- copyright notice.
-- This source file is distributed WITHOUT ANY WARRANTY, without even the implied
-- warranty of merchantability or fitness for a particular purpose.
-- By providing this source file Shimafuji Electric Inc. making no representation
-- that this source file or products generated by this source file is free from
-- any claims of infringement.
--

----------------------------------------------------------------------------------
-- spacewire link interface (implementation of chapter 8 fig.19 block diagram)
----------------------------------------------------------------------------------
---- Update history ----
-- IP-Rev1.01 2009/04/15 Correction related to timecode.
--
-- IP-Rev1.03 2010/12/21 add rx_time_counter_out_reg latch to above timecode logic.
--
-- IP-Rev1.04 2011/04/06 Changes to the "spw_if.vhd".
--
 
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity spw_link_if is
	Port (
		clock: in std_logic;
		reset: in std_logic;
		-- state machine
		tx_clock: in std_logic;
		link_start: in std_logic;
		link_disable: in std_logic;
		auto_start: in std_logic;
		link_status : out std_logic_vector (7 downto 0);
		error_status : out std_logic_vector (7 downto 0);
		reset_out : out std_logic;
		fifo_prepared : in std_logic;
		-- transmitter
		tick_in: in std_logic;
		time_in: in std_logic_vector (5 downto 0);
		control_flags_in: in std_logic_vector (1 downto 0);
		tx_write: in std_logic;
		tx_data: in std_logic_vector (7 downto 0);
		tx_control_flag: in std_logic;
		tx_ready: out std_logic;
		txclk_sel : in std_logic_vector(5 downto 0);

		-- Ver1.01 comment out port top
		credit_count : out std_logic_vector (5 downto 0);
		outstanding_count : out std_logic_vector (5 downto 0);
		-- Ver1.01 comment out port end

		-- receiver
		rx_clock: in std_logic;
		ds_clock: out std_logic;
		tick_out: out std_logic;
		time_out: out std_logic_vector (5 downto 0);
		control_flags_out: out std_logic_vector (1 downto 0);
		buffer_write: out std_logic;
		rx_data: out std_logic_vector (7 downto 0);
		rx_control_flag: out std_logic;
		buffer_ready: in std_logic;
		rx_fifo_count : in std_logic_vector(5 downto 0);
		-- serial i/o
		dout: out std_logic;
		sout: out std_logic;
		din: in std_logic;
		sin: in std_logic
	);
end spw_link_if;


architecture Behavioral of spw_link_if is

	
	component spw_receiver_sync is
		generic (
			DISCON_COUNT : integer := 141		-- 141@166MHz=850ns
		);
		port (
			SpW_S : in  std_logic;
			SpW_D : in  std_logic;
			Rx_data : out std_logic_vector (8 downto 0);
			Rx_ds : out std_logic;
			Rx_time : out std_logic_vector (7 downto 0);
			Rx_ts : out std_logic;
			Rx_Nchar : out std_logic;
			Rx_fct : out std_logic;
			Rx_null	: out std_logic;
			Rx_eep : out std_logic;
			Rx_eop : out std_logic;
			Rx_off : out std_logic;
			Credit_error : out std_logic;
			Rx_error : out std_logic;
			parity_error : out std_logic;		-- debug
			escape_error : out std_logic;		-- debug
			disconnect_error : out std_logic;	-- debug
			buf_write : out std_logic;
			buf_rdy : in std_logic;
			enable_rx : in std_logic;
			Rx_reset : in  std_logic;
			clock : in std_logic
		);
	end component;
	
	component spw_transmitter
		port (
			clk: in std_logic;
			reset: in std_logic;
			dout: out std_logic;
			sout: out std_logic;
			tick_in: in std_logic;
			time_in: in std_logic_vector (5 downto 0);
			ctrl_flg_in: in std_logic_vector (1 downto 0);
			tx_write: in std_logic;
			tx_data: in std_logic_vector (7 downto 0);
			tx_ctrl_flg: in std_logic;
			tx_rdy: out std_logic;
			enable_tx: in std_logic;
			--auto_start: in std_logic;
			send_nulls: in std_logic;
			send_fcts: in std_logic;
			send_n_chars: in std_logic;
			send_time_codes: in std_logic;
			got_fct : in std_logic;
			got_n_char : in std_logic;
			rx_fifo_count : in std_logic_vector(5 downto 0);
			credit_error : out std_logic;
			clksel : in std_logic_vector(5 downto 0);

			-- Ver1.01 comment out port top
			credit_count_out : out std_logic_vector (5 downto 0);
			outstanding_count_out : out std_logic_vector (5 downto 0);
			-- Ver1.01 comment out port end

			reset_out : in std_logic
		);
	end component;
	
	component spw_state_machine
		port (
			clk: in std_logic;
			reset: in std_logic;
			after128: in std_logic;
			after64: in std_logic;
			link_start: in std_logic;
			link_disable: in std_logic;
			auto_start : in std_logic;
			enable_tx: out std_logic;
			send_nulls: out std_logic;
			send_fcts: out std_logic;
			send_n_chars: out std_logic;
			send_time_codes: out std_logic;
			got_fct: in std_logic;
			got_time_code: in std_logic;
			got_n_char: in std_logic;
			got_null: in std_logic;
			got_bit: in std_logic;
			credit_err: in std_logic;
			rx_err: in std_logic;
			enable_rx: out std_logic;
			sequence_error : out std_logic;
			reset_out: out std_logic;
			fifo_prepared : in std_Logic;
			reset64 : out std_logic;
			count128 : out std_logic
		);
	end component;
	
	component spw_timer is
		generic (
			COUNT6US4 : integer := 640;		-- 640@100MHz=6.4us
			COUNT12US8 : integer := 1280	-- 1280@100MHz=12.8us
		);
		port (
			clk : in std_logic;
			reset : in std_logic;
			reset64 : in std_logic;
			start128 : in std_logic;
			after64us : out std_logic;
			after128us : out std_logic
		);
	end component;

	--Ver1.01 add component top
	component sync_one_pulse is
		port (
			clk : in std_logic;
			reset : in std_logic;
			async_in : in std_logic;
			sync_out : out std_logic
		);
	end component;
	--Ver1.01 add component end
	
	signal rx_clk : std_logic;
	--signal tick_out_sig : std_logic;
	--signal time_out_sig : std_logic_vector (5 downto 0);
	--signal ctrl_flg_out : std_logic_vector (1 downto 0);
	signal buf_write : std_logic;
	signal rx_ctrl_flg : std_logic;
	signal buf_rdy : std_logic;
	signal got_fct : std_logic;
	signal got_time_code : std_logic;
	signal got_n_char : std_logic;
	signal got_null : std_logic;
	signal got_bit : std_logic;
	signal credit_error : std_logic;
	signal parity_error : std_logic;
	signal escape_error : std_logic;
	signal disconnect_error : std_logic;
	signal rx_err : std_logic;
	signal enable_rx : std_logic;

	signal send_n_chars : std_logic;
	signal send_time_codes : std_logic;
	
	signal after128 : std_logic;
	signal after64 : std_logic;
	signal enable_tx : std_logic;
	signal send_nulls : std_logic;
	signal send_fcts : std_logic;
	signal reset_out_sig : std_logic;
	signal sequence_error : std_logic;

	signal reset64 : std_logic;
	signal count128 : std_logic;
	
	signal rx_off : std_logic;
	
	--Ver1.01 add signal top
	signal rx_time_counter_out : std_logic_vector(7 downto 0) := x"00";
	signal rx_tick_out : std_logic := '0';
	signal control_flags : std_logic_vector(1 downto 0) := "00";
	signal time_counter : std_logic_vector(5 downto 0) := "000000";
	signal time_counter_plus1 : std_logic_vector(5 downto 0) := "000001";
	signal internal_tick_out : std_logic := '0';
	signal got_time_code_sync : std_logic := '0';
	--Ver1.01 add signal end
	signal rx_time_counter_out_reg : std_logic_vector (7 downto 0);
	
begin

	buffer_write <= buf_write;
	buf_rdy <= buffer_ready;
	
	receiver : spw_receiver_sync
	port map (
		clock => rx_clock,					-- in
		SpW_D => din,						-- in
		SpW_S => sin,						-- in
		Rx_data (7 downto 0) => rx_data,	-- out
		Rx_data (8) => rx_control_flag,		-- out
		--tick_out => tick_out,				-- out

		-- Ver1.01 comment out port top
--		Rx_time (5 downto 0) => time_out,	-- out
--		Rx_time (7 downto 6) => control_flags_out,	-- out
		-- Ver1.01 comment out port end

		Rx_time => rx_time_counter_out,	-- out
		buf_write => buf_write,				-- out
		buf_rdy => buf_rdy,					-- in
		Rx_fct => got_fct,					-- out
		Rx_ts => got_time_code,				-- out
		Rx_Nchar => got_n_char,				-- out
		Rx_null => got_null,				-- out
		Rx_off => rx_off,					-- out
		--Credit_error => credit_error,		-- out
		Rx_error => rx_err,					-- out
		parity_error => parity_error,		-- out
		escape_error => escape_error,		-- out
		disconnect_error => disconnect_error,	-- out
		enable_rx => enable_rx,				-- in
		Rx_reset => reset_out_sig			-- in
	);
	got_bit <= not rx_off;

	--Ver1.01 comment out port top
	--tick_out <= got_time_code;
	--Ver1.01 comment out port end

	
	transmitter : spw_transmitter
	port map (
		clk => tx_clock,
		reset => reset,
		dout => dout,
		sout => sout,
		tick_in => tick_in,				-- in
		time_in => time_in,				-- in
		ctrl_flg_in => control_flags_in,	-- in
		tx_write => tx_write,			-- in
		tx_data => tx_data,				-- in
		tx_ctrl_flg => tx_control_flag,	-- in
		tx_rdy => tx_ready,				-- out
		enable_tx => enable_tx,			-- in
		--auto_start => '0',	-- in
		send_nulls => send_nulls,	-- in
		send_fcts => send_fcts,		-- in
		send_n_chars => send_n_chars,
		send_time_codes => send_time_codes,
		got_fct => got_fct,
		got_n_char => got_n_char,
		rx_fifo_count => rx_fifo_count,
		credit_error => credit_error,
		clksel => txclk_sel,

		-- Ver1.01 comment out top
		credit_count_out => credit_count,
		outstanding_count_out => outstanding_count,
		-- Ver1.01 comment out end

		reset_out => reset_out_sig			-- in
	);
	
	
	controller : spw_state_machine
	port map (
		clk => clock,
		reset => reset,
		after128 => after128,			-- in
		after64 => after64,				-- in
		link_start => link_start,		-- in
		link_disable => link_disable,	-- in
		auto_start => auto_start,		-- in
		enable_tx => enable_tx,		-- out
		send_nulls => send_nulls,	-- out
		send_fcts => send_fcts,		-- out
		send_n_chars => send_n_chars,	-- out
		send_time_codes => send_time_codes,	-- out
		got_fct => got_fct,		-- in
		got_time_code => got_time_code,	--'0',	-- in
		got_n_char => got_n_char,		-- in
		got_null => got_null,		-- in
		got_bit => '0',		-- in
		credit_err => credit_error,	-- in
		rx_err => rx_err,		-- in
		enable_rx => enable_rx,	-- out
		sequence_error => sequence_error, --out
		reset_out => reset_out_sig,	--out
		fifo_prepared => fifo_prepared,	-- in
		reset64 => reset64,		--out
		count128 => count128	-- out
	);
	
	
	timer : spw_timer
	generic map (
		COUNT6US4 => 640,	--SpaceCube=320 / SpaceWire Board=640
		COUNT12US8 => 1280	--SpaceCube=640 / SpaceWire Board=1280
	)
	port map (
		clk => clock,
		reset => reset,
		reset64 => reset64,
		start128 => count128,
		after64us => after64,
		after128us => after128
	);
	
	
	link_status (0) <= enable_tx;
	link_status (1) <= enable_rx;
	link_status (2) <= send_nulls;
	link_status (3) <= send_fcts;
	link_status (4) <= send_n_chars;
	link_status (5) <= send_time_codes;
	link_status (6) <= '0';
	link_status (7) <= reset_out_sig;	--'0';
	
	error_status (0) <= sequence_error;	--sequence
	error_status (1) <= credit_error;	--credit
	error_status (2) <= rx_err;			--rx_err(=parity, discon or escape error)
	error_status (3) <= '0';
	error_status (4) <= parity_error;		--parity
	error_status (5) <= disconnect_error;	--disconnect
	error_status (6) <= escape_error;		--escape
	error_status (7) <= '0';
	
	reset_out <= reset_out_sig;

	ds_clock <= din xor sin;
	
	-- Ver1.01 add process top	
	time_out <= time_counter;
	control_flags_out <= control_flags;
	tick_out <= internal_tick_out;
	
	time_code_sync : sync_one_pulse
	port map (
		clk => clock,
		reset => reset,
		async_in => got_time_code,
		sync_out => got_time_code_sync
	);

	TimeCode_process : process (clock,reset)
	begin
		if (reset='1') then
			time_counter <= (others => '0');
			time_counter_plus1 <= "000001";
			internal_tick_out <= '0';
			rx_time_counter_out_reg <= (others => '0');
		else
			if (clock'event and clock='1') then
				if (got_time_code_sync='1') then
					control_flags <= rx_time_counter_out_reg (7 downto 6);
					time_counter <= rx_time_counter_out_reg (5 downto 0);
					time_counter_plus1 <= rx_time_counter_out_reg (5 downto 0) + 1;
					if (time_counter_plus1 = rx_time_counter_out_reg (5 downto 0)) then
						internal_tick_out <= '1';
					end if;
				else
					internal_tick_out <= '0';
				end if;
				rx_time_counter_out_reg <= rx_time_counter_out;
			end if;
		end if;
	end process;
	-- Ver1.01 add process end
	
end Behavioral;

