--
-- Copyright(C) 2007-2010 Shimafuji Electric Inc. All Rights reserved.
--
-- Shimafuji Electric Inc. grants permission to use, copy, redistribute and/or modify
-- this souce file or products generated by this source file without restriction
-- provided that above copyright statement, this list of conditions and the following
-- disclaimer is preserved and that any derivative work contains the original
-- copyright notice.
-- This source file is distributed WITHOUT ANY WARRANTY, without even the implied
-- warranty of merchantability or fitness for a particular purpose.
-- By providing this source file Shimafuji Electric Inc. making no representation
-- that this source file or products generated by this source file is free from
-- any claims of infringement.
--
---- Update history ----
-- IP-Rev1.03 2010/12/21 (bug fix.) reset 12.8us timer when error-reset condition occured.
--
-- IP-Rev1.04 2011/04/06 Changes to the "spw_if.vhd".
--

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity spw_timer is
    generic (
		COUNT6US4 : integer := 640;
		COUNT12US8 : integer := 1280
	);
	port (
		clk : in std_logic;
		reset : in std_logic;
		reset64 : in std_logic;
		start128 : in std_logic;
		after64us : out std_logic;
		after128us : out std_logic
	);
end spw_timer;

architecture Behavioral of spw_timer is

	signal timer_state128 : std_logic;	-- off/on
	signal timer_cnt64 : std_logic_vector (9 downto 0);		-- 320@50MHz=6.4us
	signal timer_cnt128 : std_logic_vector (10 downto 0);	-- 640@50MHz=12.8us
	
begin

	ctrl_timer64 : process (clk, reset, reset64)
	begin
		if (reset = '1' or reset64 = '1') then
			timer_cnt64 <= (others => '0');
			after64us <= '0';
			
		elsif (clk'event and clk = '1') then
		
			if (timer_cnt64 < COUNT6US4) then
				timer_cnt64 <= timer_cnt64 + 1;
				after64us <= '0';
			else
				timer_cnt64 <= (others => '0');
				after64us <= '1';
			end if;
			
		end if;
	end process;


	ctrl_timer128 : process (clk, reset, start128)
	begin
		if (reset = '1' or reset64 = '1') then
			timer_state128 <= '0';
			timer_cnt128 <= (others => '0');
			after128us <= '0';
			
		elsif (clk'event and clk = '1') then
		
			if (timer_state128 = '0') then
				after128us <= '0';
				if (start128 = '1') then
					timer_state128 <= '1';
				end if;
			else
				if (timer_cnt128 < COUNT12US8) then
					timer_cnt128 <= timer_cnt128 + 1;
					after128us <= '0';
				else
					timer_cnt128 <= (others => '0');
					timer_state128 <= '0';
					after128us <= '1';
				end if;
				
			end if;
		end if;
	
	end process;
	
	
end Behavioral;

