--
-- Copyright(C) 2007-2010 Shimafuji Electric Inc. All Rights reserved.
--
-- Shimafuji Electric Inc. grants permission to use, copy, redistribute and/or modify
-- this souce file or products generated by this source file without restriction
-- provided that above copyright statement, this list of conditions and the following
-- disclaimer is preserved and that any derivative work contains the original
-- copyright notice.
-- This source file is distributed WITHOUT ANY WARRANTY, without even the implied
-- warranty of merchantability or fitness for a particular purpose.
-- By providing this source file Shimafuji Electric Inc. making no representation
-- that this source file or products generated by this source file is free from
-- any claims of infringement.
--

-- Update History --

-- IP-Rev1.02 --
-- 2010/03/02 (bug fix.) one_null_sendtOAnullȂƂ1MĂłȂ
-- 		FCT𑗐MȂ悤ɏC(StartedԂZƂ(1,2clk)̕sΉ)
-- 2010/04/09 (bug-fix.) avoid simultaneous d/s transition to 0 at error-reset.
--		separate reset input signal to hw-reset(reset) and controller_reset(reset_out).
-- IP-Rev1.03 --
-- 2010/12/21 (bug fix.) modified tx clock divider to see clksel signal only when divider count is 0
--		(fixed bug that divider counter happen to count up to max.(63) when clksel value changes)
-- 2010/12/21 (bug fix.) force txclk_sel value to default(10M) if state is not in Run state.
--
-- IP-Rev1.04 --
-- 2011/04/06 Changes to the "spw_if.vhd".
--

library IEEE; 
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity spw_transmitter is
	Port (
		clk : in std_logic;
		reset : in std_logic;
		dout : out std_logic;
		sout : out std_logic;
		tick_in : in std_logic;
		time_in : in std_logic_vector (5 downto 0);
		ctrl_flg_in : in std_logic_vector (1 downto 0);
		tx_write : in std_logic;
		tx_data : in std_logic_vector (7 downto 0);
		tx_ctrl_flg : in std_logic;
		tx_rdy : out std_logic;
		enable_tx : in std_logic;
		--auto_start : in std_logic;	-- auto_start cannot be here. (???)
		send_nulls: in std_logic;
		send_fcts: in std_logic;
		send_n_chars: in std_logic;
		send_time_codes: in std_logic;
		
		--tx_fct : in std_logic;
		got_fct : in std_logic;
		got_n_char : in std_logic;
		rx_fifo_count : in std_logic_vector(5 downto 0);
		credit_error : out std_logic;
		
		clksel : in std_logic_vector(5 downto 0);
		credit_count_out : out std_logic_vector (5 downto 0);
		outstanding_count_out : out std_logic_vector (5 downto 0);
		reset_out: in std_logic
	);
end spw_transmitter;

architecture Behavioral of spw_transmitter is

	
	component sync_one_pulse is
		port (
			clk : in std_logic;
			reset : in std_logic;
			async_in : in std_logic;
			sync_out : out std_logic
		);
	end component;
	
	
	type txstate is (
		TXS_STOP,
		TXS_PARITY,
		TXS_CTRL,
		TXS_DATA
	);
	signal tx_state : txstate;
	
	signal div_count : std_logic_vector (5 downto 0);
	signal div_state : std_logic;
	
	signal tx_parity : std_logic;
	signal null_send : std_logic;
	signal tcode_send : std_logic;
	signal d_out_reg : std_logic;
	signal s_out_reg : std_logic;
	signal send_start: std_logic;
	signal send_done : std_logic;
	signal send_data : std_logic_vector (12 downto 0);
	signal send_num : std_logic_vector (3 downto 0);
	
	signal tx_write_sync : std_logic;
	signal decriment_credit : std_logic;
	
	signal tx_fct_start : std_logic;
	signal tx_fct_done : std_logic;
	
	signal tick_in_sync : std_logic;
	signal tx_time_start : std_logic;
	signal tx_time_done : std_logic;
	signal tx_time_state : std_logic;
	
	signal got_n_char_sync : std_logic;
	signal got_n_char_sync_delay : std_logic_vector (9 downto 0);
	signal outstanding_count : std_logic_vector (5 downto 0);
	signal rx_fifo_cnt_buf0 : std_logic_vector (5 downto 0);
	signal rx_fifo_cnt_buf1 : std_logic_vector (5 downto 0);
	signal rx_fifo_cnt_buf : std_logic_vector (5 downto 0);
	signal tx_fct_state : std_logic;
	
	signal tx_data_buf : std_logic_vector (7 downto 0);
	signal tx_ctrl_flg_buf : std_logic;
	
	signal got_fct_sync : std_logic;
	signal rx_fifo_space_adv : std_logic_vector (5 downto 0);
	signal tx_credit_count : std_logic_vector (6 downto 0);
	signal credit_error_n : std_logic;
	signal credit_error_f : std_logic;
	
	--20081126 Takayuki Yuasa modified
	signal time_in_latched : std_logic_vector (5 downto 0) := "000000";
	
	signal one_null_send : std_logic;
	
	signal reset_in : std_logic;
	
	-- Ver1.04
	constant INIT_TXCLK_SEL : std_logic_vector (5 downto 0) := "001001";		-- 10M
	signal clksel_reg : std_logic_vector (5 downto 0);
	signal clksel_cur : std_logic_vector (5 downto 0);
	
begin

	reset_in <= reset or reset_out;
	
	-- synchronize to tx_clock
	
	tx_write_pulse : sync_one_pulse
	port map (
		clk => clk,
		reset => reset_in,
		async_in => tx_write,
		sync_out => tx_write_sync
	);
	
	tick_in_pulse : sync_one_pulse
	port map (
		clk => clk,
		reset => reset_in,
		async_in => tick_in,
		sync_out => tick_in_sync
	);
	
	got_fct_pulse : sync_one_pulse
	port map (
		clk => clk,
		reset => reset_in,
		async_in => got_fct,
		sync_out => got_fct_sync
	);
	
	
	process (clk, reset_in)
	begin
		if (reset_in = '1') then
			tx_time_state <= '0';
			tx_time_start <= '0';
			
		elsif (clk'event and clk = '1') then
			if (tx_time_state = '0') then
				if (tick_in_sync = '1') then
					tx_time_start <= '1';
					tx_time_state <= '1';
					--20081126 Takayuki Yuasa modified
					time_in_latched <= time_in;
				end if;
			else
				if (tx_time_done = '1') then
					tx_time_start <= '0';
					tx_time_state <= '0';
				end if;
			end if;
		end if;
	
	end process;
	
	--
	-- tx credit count manage
	--
	credit_error <= credit_error_n or credit_error_f;
	
	credit_count_out <= tx_credit_count (5 downto 0);
	outstanding_count_out <= outstanding_count;
	
	process (clk, reset_in)
	begin
		if (reset_in = '1') then
			tx_credit_count <= (others => '0');
			credit_error_f <= '0';
			
		elsif (clk'event and clk = '1') then
			if (got_fct_sync = '1') then
				if (decriment_credit = '1') then
					tx_credit_count <= tx_credit_count + 7;
				else
					tx_credit_count <= tx_credit_count + 8;
				end if;
			elsif (decriment_credit = '1') then
				if (tx_credit_count /= "0000000") then
					tx_credit_count <= tx_credit_count - 1;
				end if;
			else
				if (tx_credit_count > "00111000") then
					credit_error_f <= '1';
				else
					credit_error_f <= '0';
				end if;
			
			end if;
		end if;
	
	end process;
	
	
	-- rx credit count manage
	-- fct transmit handshake
	--
	got_n_char_pulse : sync_one_pulse
	port map (
		clk => clk,
		reset => reset_in,
		async_in => got_n_char,
		sync_out => got_n_char_sync
	);
	
	process (clk, reset_in)
	begin
		if (reset_in = '1') then
			got_n_char_sync_delay <= (others => '0');
		elsif (clk'event and clk = '1') then
			got_n_char_sync_delay (0) <= got_n_char_sync;
			got_n_char_sync_delay (9 downto 1) <= got_n_char_sync_delay (8 downto 0);
		end if;
	end process;
	
	process (clk, reset_in)
	begin
		if (reset_in = '1') then
			rx_fifo_cnt_buf0 <= (others => '0');
			rx_fifo_cnt_buf1 <= (others => '0');
			rx_fifo_cnt_buf <= (others => '0');
			
		elsif (clk'event and clk = '1') then
			rx_fifo_cnt_buf0 <= rx_fifo_count;
			rx_fifo_cnt_buf1 <= rx_fifo_cnt_buf0;
			
			if (rx_fifo_cnt_buf1 = rx_fifo_cnt_buf0) then
				rx_fifo_cnt_buf <= rx_fifo_cnt_buf1;
			end if;
		end if;
	
	end process;

	process (clk, reset_in)
	begin
		if (reset_in = '1') then
			outstanding_count <= "000000";
			tx_fct_state <= '0';
			tx_fct_start <= '0';
			credit_error_n <= '0';
			
		elsif (clk'event and clk = '1') then
	--		if (tx_fct_state = '0') then
	--			if (outstanding_count + rx_fifo_cnt_buf <= "110000") then
	--				if (got_n_char_sync_delay (9) = '1') then
	--					outstanding_count <= outstanding_count + 7;
	--				else
	--					outstanding_count <= outstanding_count + 8;
	--				end if;
	--				tx_fct_start <= '1';
	--				tx_fct_state <= '1';
	--			else
	--				if (got_n_char_sync_delay (9) = '1') then
	--					outstanding_count <= outstanding_count - 1;
	--				end if;
	--			end if;
	--		else
	--			if (tx_fct_done = '1') then
	--				tx_fct_start <= '0';
	--				tx_fct_state <= '0';
	--			end if;
	--			if (got_n_char_sync_delay (9) = '1') then
	--				outstanding_count <= outstanding_count - 1;
	--			end if;
	--		end if;
			
			if (tx_fct_state = '0') then
				if (outstanding_count + rx_fifo_cnt_buf <= "110000") then
					tx_fct_start <= '1';
					tx_fct_state <= '1';
				end if;
				if (got_n_char_sync_delay (9) = '1') then
					outstanding_count <= outstanding_count - 1;
				end if;
			else
				if (tx_fct_done = '1') then
					if (got_n_char_sync_delay (9) = '1') then
						outstanding_count <= outstanding_count + 7;
					else
						outstanding_count <= outstanding_count + 8;
					end if;
					tx_fct_start <= '0';
					tx_fct_state <= '0';
				else
					if (got_n_char_sync_delay (9) = '1') then
						outstanding_count <= outstanding_count - 1;
					end if;
				end if;
			end if;
			
			--
			-- creadit error o͂ɒu
			--
			if (got_n_char_sync_delay (9) = '1' and outstanding_count = "000000") then
				credit_error_n <= '1';
			else
				credit_error_n <= '0';
			end if;
			
		end if;
	
	end process;
	
	
	-- data transmit handshake
	
	process (clk, reset_in)
	begin
		if (reset_in = '1') then
			send_start <= '0';
			tx_data_buf <= (others => '0');
			tx_ctrl_flg_buf <= '0';
			
		elsif (clk'event and clk = '1') then
			if (tx_write_sync = '1') then
				tx_data_buf <= tx_data;
				tx_ctrl_flg_buf <= tx_ctrl_flg;
				send_start <= '1';
			elsif (send_done = '1') then
				send_start <= '0';
			end if;
		end if;
	
	end process;
	
	-- Ver1.04
	-- initalally txclk_sel=9(10M), then it enters Run state, txclk_sel become specified rate.
	process (clk, reset) begin
		if (reset = '1') then
			clksel_reg <= INIT_TXCLK_SEL;
		elsif (clk'event and clk = '1') then
			if (send_n_chars = '1') then
				clksel_reg <= clksel;
			else
				clksel_reg <= INIT_TXCLK_SEL;
			end if;
		end if;
	end process;
	
	--
	-- transmitter logic
	--
	process (clk, reset)
	begin
		if (reset = '1') then
			div_count <= (others => '0');
			div_state <= '0';
			clksel_cur <= INIT_TXCLK_SEL;
		elsif (clk'event and clk = '1') then
			if (div_count = clksel_cur) then
				div_count <= (others => '0');
				div_state <= '1';
				clksel_cur <= clksel_reg;
			else
				div_count <= div_count + 1;
				div_state <= '0';
			end if;
		end if;
	end process;
	
	process (clk, reset)
	begin
		if (reset = '1') then
			tx_parity <= '0';
			tx_state <= TXS_STOP;
			d_out_reg <= '0';
			s_out_reg <= '0';
			null_send <= '0';
			tcode_send <= '0';
			send_done <= '0';
			tx_fct_done <= '0';
			tx_time_done <= '0';
			decriment_credit <= '0';
			one_null_send <= '0';
			
		elsif (clk'event and clk = '1') then
			if (div_state = '1') then
				if (tx_state = TXS_STOP) then
					if (enable_tx = '1' and send_nulls = '1') then
						tx_state <= TXS_PARITY;
					else
						tx_parity <= '0';
						d_out_reg <= '0';
						s_out_reg <= '0';
						null_send <= '0';
						one_null_send <= '0';
					end if;
				
				elsif (tx_state = TXS_PARITY) then
					if (enable_tx = '1') then
						if (null_send = '1') then
							-- send pending FCT of NULL(ESC+FCT)
							send_data <= "0000000000" & "001";
							send_num <= x"2";
							if (d_out_reg = tx_parity) then
								d_out_reg <= tx_parity;
								s_out_reg <= not s_out_reg;
							else
								d_out_reg <= tx_parity;
								s_out_reg <= s_out_reg;
							end if;
							null_send <= '0';
						
						elsif (tcode_send = '1') then
							-- send pending TIME of TCODE(ESC+TIME)
							--20081126 Takayuki Yuasa modified
							--send_data <= "0000" & ctrl_flg_in & time_in & '0';
							send_data <= "0000" & ctrl_flg_in & time_in_latched & '0';
							send_num <= x"8";
							if (d_out_reg = (tx_parity xor '1')) then
								d_out_reg <= tx_parity xor '1';
								s_out_reg <= not s_out_reg;
							else
								d_out_reg <= tx_parity xor '1';
								s_out_reg <= s_out_reg;
							end if;
							tcode_send <= '0';
							tx_time_done <= '1';
						
						elsif (tx_time_start = '1') then
							-- send ESC of TCODE
							send_data <= "0000000000" & "111";
							send_num <= x"2";
							if (d_out_reg = tx_parity) then
								d_out_reg <= tx_parity;
								s_out_reg <= not s_out_reg;
							else
								d_out_reg <= tx_parity;
								s_out_reg <= s_out_reg;
							end if;
							tcode_send <= '1';
							
						elsif (send_fcts = '1' and tx_fct_start = '1' and one_null_send = '1') then
							-- send ESC of NULL
							send_data <= "0000000000" & "001";
							send_num <= x"2";
							if (d_out_reg = tx_parity) then
								d_out_reg <= tx_parity;
								s_out_reg <= not s_out_reg;
							else
								d_out_reg <= tx_parity;
								s_out_reg <= s_out_reg;
							end if;
							tx_fct_done <= '1';
							
						--elsif (send_start = '1' and tx_credit_count > "001000") then
						--elsif (send_n_chars = '1' and send_start = '1' and tx_credit_count /= "000000") then
						elsif (send_n_chars = '1' and send_start = '1') then
							decriment_credit <= '1';
							if (tx_ctrl_flg_buf = '1' and tx_data_buf(0) = '0') then
								-- send EOP
								send_data <= "00000" & "00000" & "101";
								send_num <= x"2";
								if (d_out_reg = tx_parity) then
									d_out_reg <= tx_parity;
									s_out_reg <= not s_out_reg;
								else
									d_out_reg <= tx_parity;
									s_out_reg <= s_out_reg;
								end if;
							elsif (tx_ctrl_flg_buf = '1' and tx_data_buf(0) = '1') then
								-- send EEP
								send_data <= "00000" & "00000" & "011";
								send_num <= x"2";
								if (d_out_reg = tx_parity) then
									d_out_reg <= tx_parity;
									s_out_reg <= not s_out_reg;
								else
									d_out_reg <= tx_parity;
									s_out_reg <= s_out_reg;
								end if;
							else
								-- send 8-bit data
								send_data <= "0000" & tx_data_buf & '0';
								send_num <= x"8";
								if (d_out_reg = (tx_parity xor '1')) then
									d_out_reg <= tx_parity xor '1';
									s_out_reg <= not s_out_reg;
								else
									d_out_reg <= tx_parity xor '1';
									s_out_reg <= s_out_reg;
								end if;
							end if;
							send_done <= '1';
							
						elsif (send_nulls = '1') then
							-- send ESC of NULL
							send_data <= "0000000000" & "111";
							send_num <= x"2";
							if (d_out_reg = tx_parity) then
								d_out_reg <= tx_parity;
								s_out_reg <= not s_out_reg;
							else
								d_out_reg <= tx_parity;
								s_out_reg <= s_out_reg;
							end if;
							null_send <= '1';
							one_null_send <= '1';
						end if;
						tx_state <= TXS_CTRL;
					else
						d_out_reg <= '0';
						tx_state <= TXS_STOP;
					end if;
				
				elsif (tx_state = TXS_CTRL) then
					if (enable_tx = '1') then
						send_done <= '0';
						decriment_credit <= '0';
						tx_fct_done <= '0';
						tx_time_done <= '0';
						send_num <= send_num - 1;
						if (d_out_reg = send_data(0)) then
							d_out_reg <= send_data(0);
							s_out_reg <= not s_out_reg;
						else
							d_out_reg <= send_data(0);
							s_out_reg <= s_out_reg;
						end if;
						send_data <= '0' & send_data(12 downto 1);
						tx_parity <= '0';
						tx_state <= TXS_DATA;
					else
						d_out_reg <= '0';
						tx_state <= TXS_STOP;
					end if;
				
				elsif (tx_state = TXS_DATA) then
					if (enable_tx = '1') then
						if (d_out_reg = send_data(0)) then
							d_out_reg <= send_data(0);
							s_out_reg <= not s_out_reg;
						else
							d_out_reg <= send_data(0);
							s_out_reg <= s_out_reg;
						end if;
						tx_parity <= tx_parity xor send_data(0);
						send_data <= '0' & send_data(12 downto 1);
						
						if (send_num = "0000") then
							tx_state <= TXS_PARITY;
						else
							send_num <= send_num - 1;
						end if;
					else
						d_out_reg <= '0';
						tx_state <= TXS_STOP;
					end if;
				
				end if;
			else
				tx_fct_done <= '0';
				send_done <= '0';
				decriment_credit <= '0';
			end if;
		end if;
	end process;
	
	--
	-- outputs
	--
	tx_rdy <= '0' when (send_start = '1' or tx_fct_start = '1' or tx_credit_count = "0000000") else '1';
	
	dout <= d_out_reg;
	sout <= s_out_reg;
	
end Behavioral;
