--
-- Copyright(C) 2007-2010 Shimafuji Electric Inc. All Rights reserved.
--
-- Shimafuji Electric Inc. grants permission to use, copy, redistribute and/or modify
-- this souce file or products generated by this source file without restriction
-- provided that above copyright statement, this list of conditions and the following
-- disclaimer is preserved and that any derivative work contains the original
-- copyright notice.
-- This source file is distributed WITHOUT ANY WARRANTY, without even the implied
-- warranty of merchantability or fitness for a particular purpose.
-- By providing this source file Shimafuji Electric Inc. making no representation
-- that this source file or products generated by this source file is free from
-- any claims of infringement.
--
---- Update history ----
--
-- IP-Rev1.04 2011/04/06 Changes to the "spw_if.vhd".
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

--library unisim;
--use unisim.vcomponents.all;

entity sync_one_pulse is
	port (
		clk : in std_logic;
		reset : in std_logic;
		async_in : in std_logic;
		sync_out : out std_logic
	);
end sync_one_pulse;

architecture Behavioral of sync_one_pulse is

	signal latched_async : std_logic;
	signal sync_reg : std_logic;
	signal sync_clr : std_logic;
	
begin

	process (async_in, reset, sync_clr) begin
		if (reset = '1' or sync_clr = '1') then
			latched_async <= '0';
		elsif (async_in'event and async_in = '1') then
			latched_async <= '1';
		end if;
	end process;

	process (clk, reset, sync_clr) begin
		if (reset = '1' or sync_clr = '1') then
			sync_reg <= '0';
		elsif (clk'event and clk = '1') then
			if (latched_async = '1') then
				sync_reg <= '1';
			end if;
		end if;
	end process;
	
	process (clk, reset, sync_reg) begin
		if (reset = '1') then
			sync_out <= '0';
			sync_clr <= '0';
		elsif (clk'event and clk = '1') then
			if (sync_reg = '1' and sync_clr = '0') then
				sync_out <= '1';
				sync_clr <= '1';
			elsif (sync_reg = '1') then
				sync_out <= '0';
				sync_clr <= '0';
			else
				sync_out <= '0';
				sync_clr <= '0';
			end if;
		end if;
	end process;
	
end Behavioral;

